#include "widget.h"
#include "ui_widget.h"
#include <QDebug>
#include <QMessageBox>
#include <QPushButton>
#include <QLabel>
#include <QSlider>
#include <QSpacerItem>
#include <QDir>
#include <QFileDialog>


#define NTEMPS 5            // le nombre de temps

//____________________________________________________________________________________________________________________________________
//                                              constructeur de la classe d'interface
//____________________________________________________________________________________________________________________________________
Son_bouton::Son_bouton(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::Son_bouton)
{
    // initialisation de l'interface
    ui->setupUi(this);

    // initialisation diverses
    temps = 0;                              // 1er temps
    ntemps = NTEMPS;                        // le nombre de temps sur l'interface

    // instanciation du timer
    timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(on_timer()));

    // choix des fichiers de son dans un répertoire
    QStringList soundNames = QFileDialog::getOpenFileNames(this, "Fichier des sons", getenv("$HOME"), "*.wav | *.mp3");
    nsounds = soundNames.size();                       // le nombre de fichiers ---> nombre de players, de boutton, de slider, .etc.

    // lecture des différents sons dans un vecteur (tableau) de QMediaContents
    for (int i=0 ; i<nsounds ; i++)
    sounds.push_back(QMediaContent(QUrl::fromLocalFile(soundNames[i])));

    // les différents players et leurs sons respectifs (liste à deux dimensions)
    for (int j=0 ; j<ntemps ; j++){
        QVector <QMediaPlayer*> lplayer;        // une colonne de players par temps contenant tous les sons
        for (int i=0 ; i<nsounds ; i++){
         lplayer.push_back(new QMediaPlayer());
        lplayer[i]->setMedia(sounds[i]);
        lplayer[i]->setPlaybackRate(1.0);
        }
    players.push_back(lplayer);                 // on ajoute la colonne
    }

    // construction de l'interface
    setLayout(ui->gridLayout);      // l'interface su présente en grille (lignes x colonnes) ajustée à la fenêre prinicpale

    // pour tous les sons, tous les temps
    for (int i=0 ; i<nsounds ; i++){
        // un label pour le nom du fichier
        QLabel *l = new QLabel(soundNames[i]);          // le nom du fichier dans le label
        l->setMaximumHeight(20);
        ui->gridLayout->addWidget(l, i, 0, Qt::AlignHCenter | Qt::AlignVCenter);             // ajout à la ligne i, colonne 0

        // une ligne de checkbox représentant un son, 1 par temps
        QVector <QCheckBox *> lcheckboxes;          // une ligne de checkbox

        // les checkboxes pour chache temps
        for (int j=0 ; j<ntemps ; j++){
        // un checkbox
        lcheckboxes.push_back(new QCheckBox(this));
        lcheckboxes[j]->setStyleSheet("QCheckBox::indicator { width:50px; height: 50px;}");     // la feuille de style
        lcheckboxes[j]->setChecked(false);                                                      // pas coché
        lcheckboxes[j]->setObjectName("checkbox_"+QString().setNum(i)+"_"+QString().setNum(j));          // nom de l'objet
        lcheckboxes[j]->setText("");                                                             // pas de texte
        ui->gridLayout->addWidget(lcheckboxes[j], i, j+1, Qt::AlignHCenter | Qt::AlignVCenter);
        }
    this->checkboxes.push_back(lcheckboxes);        // on push la ligne
    }

    // les sliders permettant de régler la durée des temps
    for (int j=0 ; j<ntemps ; j++){
        // les slider de tempo
        tempoSliders.push_back(new QSlider(this));
        tempoSliders[j]->setObjectName("tempo"+QString().setNum(j));
        tempoSliders[j]->setOrientation(Qt::Horizontal);
        tempoSliders[j]->setMinimum(1);
        tempoSliders[j]->setMaximum(1000);
        tempoSliders[j]->setValue(500);
        ui->gridLayout->addWidget(tempoSliders[j], nsounds, j+1, Qt::AlignHCenter | Qt::AlignVCenter);

        // les radiobuttons d'avancement, matérialise les temps
        progressRadioButtons.push_back(new QRadioButton(this));
        progressRadioButtons[j]->setText("");
        progressRadioButtons[j]->setChecked(false);
        ui->gridLayout->addWidget(progressRadioButtons[j], nsounds+1, j+1, Qt::AlignHCenter);

    }

    // le timer fonctionnent en single shot
    timer->setInterval(tempoSliders[temps]->value());
    timer->setSingleShot(true);
    timer->start();

}

//____________________________________________________________________________________________________________________________________
//                                      destructeur de la classe d'interface
//____________________________________________________________________________________________________________________________________
Son_bouton::~Son_bouton()
{
    delete ui;
    // on ne libère pas tout, on fait confiance au ramasse miette!
}


//____________________________________________________________________________________________________________________________________
//                                                  le slot du timer
//____________________________________________________________________________________________________________________________________
void Son_bouton::on_timer(){
    // pour le temps courant
    progressRadioButtons[temps]->setChecked(true);              // on visialise la position du temps sur des radiobuttons
    // pour tous les sons
    for (int i=0 ; i<nsounds ; i++)
        if (checkboxes[i][temps]->isChecked()) {                // s'il est coché, on le joue
            players[temps][i]->setPosition(0);
            players[temps][i]->play();
        }
    // on ajuste le temps au slider correspondant
    timer->setInterval(tempoSliders[temps]->value());
    timer->start();
    // on passe au temps suivant
    temps = (temps+1)%ntemps;
}

